'this script parses the file STPatchAssessment.xml and lists all missing hotfixes after an Altiris PatchAssessment
'09.02.2013, carsten_giese@symantec.com

setlocale(1031)

const HKLM = &H80000002
Set reg=GetObject("winmgmts:!\\.\root\default:StdRegProv")
Set fso = CreateObject("Scripting.FileSystemObject")

'get SMP servername from registry:
result = Reg.GetStringValue(HKLM,"Software\Altiris\Altiris Agent\Servers","",SMP)
if result<>0 then 
	wscript.echo "No Altiris Agent installed."
	wscript.quit
end if
Reg.GetStringValue HKLM,"Software\Altiris\Altiris Agent\Servers\" & SMP,"Web",SMPweb
Reg.GetStringValue HKLM,"Software\Altiris\Altiris Agent","MachineGUID",guid

'read the XML file from last assessment:
reg.GetStringValue HKLM,"Software\Altiris\Altiris Agent","InstallDir",dir
dir = dir & "\Agents\SoftwareManagement\Software Delivery\{6D417916-467C-46A7-A870-6D86D9345B61}\cache"
xmlfile = dir & "\STPatchAssessment.xml"

set xmldoc = CreateObject("Microsoft.XMLDOM")
xmldoc.async=false

scandate=now()
if fso.FileExists(xmlfile) then
	xmldoc.load(xmlfile)
	for each p in xmldoc.SelectNodes("//PatchScan")
		scanDate = p.Attributes.getNamedItem("ScanDate").Text
	next
end if
scanAgeInMin = DateDiff("n",cdate(scandate),now())

'check, if we have to start the assessment first:
if not fso.FileExists(xmlfile) or scanAgeInMin > 24*60 then 
	set client = wscript.createobject("Altiris.AeXNSClient") 
	set swd = client.ClientPolicyMgr.ClientAgent("Altiris.SWD")
	wscript.echo "last assessment is older than 24h. re-scanning..."
	taskGuid = "{1CFEC044-FFF0-40D4-A2FA-F312774116B4}" 
	swd.RunAdvertisement taskGuid,1
	do
		busy=0
		set tasks = swd.Advertisements
		for each task in tasks
			if task.Program.Status>=6 then
				busy=1
				wscript.sleep 2000
				exit for
			end if
		next
	loop until busy=0
	
	'trigger the Patch Remediation schedule on the SMP:
	wscript.echo "triggering patch remediation on SMP..."  & vbCrLf
	PatchRemediationSchedule="1bf89561-394a-42e1-88b2-bae5f42874cc"
	URL = SMPweb & "/ASDK.NS/ItemManagementService.asmx/ExecuteSchedulableItem?itemGuid=" & PatchRemediationSchedule
	xmldoc.load(URL)
	wscript.echo xmldoc.xml
	
	'reload the XMLfile
	xmldoc.load(xmlfile)
end if

'show header with global scan-info:
for each p in xmldoc.SelectNodes("//PatchScan")
	with p.Attributes
		scanDate = .getNamedItem("ScanDate").Text
		scanVer  = .getNamedItem("ScannerVersion").Text
		XMLDate  = .getNamedItem("XMLRevisionDate").Text
		XMLVer   = .getNamedItem("XMLVersion").Text
	end with
next
wscript.echo "Scan Date:   " & FormatDateTime(scanDate,0)
wscript.echo "Scanner:     " & scanVer
wscript.echo "XML Date:    " & FormatDateTime(XMLDate,2)
wscript.echo "XML Version: " & XMLVer & vbCrLf

msg = ""
for each i in XMLDoc.SelectNodes("//Item")
	if i.GetAttribute("Superseded") = "false" then
		if i.GetAttribute("Status") = "Missing" then
			id = i.GetAttribute("BulletinID")
			sq = i.GetAttribute("SQNumber")
			pn = i.GetAttribute("PatchName")
			'info = getDetails(id,pn)
			msg = msg & left(id & "          ",12) & vbTab & left(sq & "          ",12) & vbTab & pn & vbCrLf
			if info <> "" then
				msg2 = msg2 & id & ", " & pn & vbCrLf & vbTab & "package: " & split(info,vbTab)(0) & vbCrLf & vbTab & "command: " & split(info,vbTab)(1) & vbCrLf
			end if
		end if
	end if
next

wscript.echo "Missing Patches on this machine at last assessment:" & vbCrLf
arrMsg = split(msg,vbCrLf)
arrMsg = bubblesort(arrMsg)
for i=0 to ubound(arrMsg)
	wscript.echo arrMsg(i)
next
wscript.echo vbCrLf

'put the SMP in the intranet zone of the current user to execute an *.asmx-page:
Const HKCU=&H80000001
regPath = "Software\Microsoft\Windows\CurrentVersion\Internet Settings\ZoneMap\Domains\" & SMP
reg.CreateKey HKCU, regPath
reg.SetDWORDValue HKCU, regPath, split(SMPweb,":")(0), 1

'run report on SMP via ASDK-call (added WBEM-timestamp to bypass the IIS-caching):
Set dateTime = CreateObject("WbemScripting.SWbemDateTime")
dateTime.SetVarDate now()
suffix = left(dateTime.value,14)

reportGUID="f08e89eb-a4d0-4172-a97e-80ae676dad12"
URL = SMPweb & "/ASDK.NS/ReportManagementService.asmx/RunReportWithParameters?reportItemGuid=" & reportGUID & "&nameValuePairs=ComputerGUID=" & GUID & "&Timestamp=" & suffix
xmldoc.load(URL)

wscript.echo "Outstanding Patch-Installations in Altiris environment:" & vbCrLf
for each t in XMLdoc.SelectNodes("//Table")
	bu = t.GetElementsByTagName("bulletin")(0).text
	fn = t.GetElementsByTagName("FileName")(0).text
	rd = t.GetElementsByTagName("ReleaseDate")(0).text
	rd = FormatDateTime(cdate(left(rd,10)),2)
	ps = t.GetElementsByTagName("PatchSize")(0).text
	wscript.echo bu & vbTab & fn & vbTab & rd & vbTab & ps
next

function bubblesort(x)
	dim i,j,tmp
	for i = 0 to ubound(x)
		for j = i + 1 to ubound(x)
			if x(i) > x(j) then
				tmp = x(i)
				x(i) = x(j)
				x(j) = tmp
			end if
		next
	next
	bubblesort = x
end function

